use beteran_common_rust as bcr;
use beteran_protobuf_rust as bpr;
use prost::Message;

///
pub struct Service {
  connection_broker: nats::asynk::Connection,
  queue_broker: String,
}

impl std::fmt::Debug for Service {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("beteran-frontend-server-edge::member_bank_account::service::Service")
      .finish()
  }
}

impl Service {
  ///
  pub fn new(connection_broker: nats::asynk::Connection, queue_broker: String) -> Service {
    Service {
      connection_broker,
      queue_broker,
    }
  }

  pub async fn subscribe(&self) -> std::result::Result<(), std::boxed::Box<dyn std::error::Error>> {
    futures::try_join!(
      self.create_member_bank_account(),
      self.get_member_bank_account(),
      self.update_member_bank_account(),
      self.update_member_bank_account_for_exchange_password(),
    )
    .map(|_| ())
  }

  fn get_client_in_header(
    &self,
    message: &nats::asynk::Message,
  ) -> Result<bpr::models::core::network::Client, bcr::error::rpc::Error> {
    match &message.headers {
      Some(headers) => {
        let client = match headers.get(bpr::c2se::core::network::HEADER_CLIENT) {
          Some(c) => {
            let msg = base64::decode(c).map_err(|e| {
              bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
                message: format!("invalid header: {}", e),
              })
            })?;
            bpr::models::core::network::Client::decode(msg.as_slice()).map_err(|e| {
              bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
                message: format!("invalid header: {}", e),
              })
            })?
          }
          None => {
            return Err(bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
              message: "invalid client information".to_string(),
            }));
          }
        };

        Ok(client)
      }
      None => Err(bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
        message: "invalid header".to_string(),
      })),
    }
  }

  async fn get_member_session_with_renewal(
    &self,
    client: &bpr::models::core::network::Client,
  ) -> Result<Option<bpr::models::member_session::MemberSession>, bcr::error::rpc::Error> {
    let ss_get_member_session_req = bpr::ss::member_session::GetMemberSessionWithRenewalRequest {
      client: Some(client.clone()),
      request: Some(bpr::ss::member_session::get_member_session_with_renewal_request::Request {}),
    };

    let ss_get_member_session_res_msg = self
      .connection_broker
      .request(
        bpr::ss::member_session::SUBJECT_GET_MEMBER_SESSION_WITH_RENEWAL,
        ss_get_member_session_req.encode_to_vec(),
      )
      .await
      .map_err(|e| {
        bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
          code: bpr::protobuf::rpc::Error::SERVER_00,
          message: format!("server {}", e),
          data: None,
        })
      })?;

    let ss_get_member_session_res =
      bpr::ss::member_session::GetMemberSessionWithRenewalResponse::decode(
        ss_get_member_session_res_msg.data.as_slice(),
      )
      .map_err(|e| {
        bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
          code: bpr::protobuf::rpc::Error::SERVER_00,
          message: format!("server {}", e),
          data: None,
        })
      })?;

    if let Some(e) = ss_get_member_session_res.error {
      return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
        code: bpr::protobuf::rpc::Error::SERVER_00,
        message: format!("server {}", e),
        data: None,
      }));
    }

    if let Some(r) = ss_get_member_session_res.result {
      return Ok(r.member_session);
    }

    Ok(None)
  }

  async fn create_member_bank_account(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::frontend::member_bank_account::SUBJECT_CREATE_MEMBER_BANK_ACCOUNT,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req = bpr::c2se::member_bank_account::CreateMemberBankAccountRequest::decode(
          message.data.as_slice(),
        )
        .map_err(|e| {
          bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
            message: format!("invalid request: {}", e),
          })
        })?;

        let ss_create_member_bank_account_req =
          bpr::ss::member_bank_account::CreateMemberBankAccountRequest {
            client: Some(client),
            request: Some(
              bpr::ss::member_bank_account::create_member_bank_account_request::Request {
                member_id: req.member_id,
                bank_id: req.bank_id,
                name: req.name,
                account_number: req.account_number,
                exchange_password: req.exchange_password,
                memo: req.memo,
              },
            ),
          };

        let ss_create_member_bank_account_res_msg = self
          .connection_broker
          .request(
            bpr::ss::member_bank_account::SUBJECT_CREATE_MEMBER_BANK_ACCOUNT,
            ss_create_member_bank_account_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_create_member_bank_account_res =
          bpr::ss::member_bank_account::CreateMemberBankAccountResponse::decode(
            ss_create_member_bank_account_res_msg.data.as_slice(),
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        if let Some(e) = ss_create_member_bank_account_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_create_member_bank_account_res.result {
          message
            .respond(
              bpr::c2se::member_bank_account::CreateMemberBankAccountResponse {
                error: None,
                result: Some(
                  bpr::c2se::member_bank_account::create_member_bank_account_response::Result {
                    member_bank_account: r.member_bank_account,
                  },
                ),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::member_bank_account::CreateMemberBankAccountResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn get_member_bank_account(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::frontend::member_bank_account::SUBJECT_GET_MEMBER_BANK_ACCOUNT,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req = bpr::c2se::member_bank_account::GetMemberBankAccountRequest::decode(
          message.data.as_slice(),
        )
        .map_err(|e| {
          bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
            message: format!("invalid request: {}", e),
          })
        })?;

        let ss_get_member_bank_account_req =
          bpr::ss::member_bank_account::GetMemberBankAccountRequest {
            client: Some(client),
            request: Some(
              bpr::ss::member_bank_account::get_member_bank_account_request::Request { id: req.id },
            ),
          };

        let ss_get_member_bank_account_res_msg = self
          .connection_broker
          .request(
            bpr::ss::member_bank_account::SUBJECT_GET_MEMBER_BANK_ACCOUNT,
            ss_get_member_bank_account_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_get_member_bank_account_res =
          bpr::ss::member_bank_account::GetMemberBankAccountResponse::decode(
            ss_get_member_bank_account_res_msg.data.as_slice(),
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        if let Some(e) = ss_get_member_bank_account_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_get_member_bank_account_res.result {
          message
            .respond(
              bpr::c2se::member_bank_account::GetMemberBankAccountResponse {
                error: None,
                result: Some(
                  bpr::c2se::member_bank_account::get_member_bank_account_response::Result {
                    member_bank_account: r.member_bank_account,
                  },
                ),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::member_bank_account::GetMemberBankAccountResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn update_member_bank_account(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::frontend::member_bank_account::SUBJECT_UPDATE_MEMBER_BANK_ACCOUNT,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req = bpr::c2se::member_bank_account::UpdateMemberBankAccountRequest::decode(
          message.data.as_slice(),
        )
        .map_err(|e| {
          bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
            message: format!("invalid request: {}", e),
          })
        })?;

        let ss_update_member_bank_account_req =
          bpr::ss::member_bank_account::UpdateMemberBankAccountRequest {
            client: Some(client),
            request: Some(
              bpr::ss::member_bank_account::update_member_bank_account_request::Request {
                id: req.id,
                bank_id: req.bank_id,
                name: req.name,
                account_number: req.account_number,
                memo: req.memo,
              },
            ),
          };

        let ss_update_member_bank_account_res_msg = self
          .connection_broker
          .request(
            bpr::ss::member_bank_account::SUBJECT_UPDATE_MEMBER_BANK_ACCOUNT,
            ss_update_member_bank_account_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_update_member_bank_account_res =
          bpr::ss::member_bank_account::UpdateMemberBankAccountResponse::decode(
            ss_update_member_bank_account_res_msg.data.as_slice(),
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        if let Some(e) = ss_update_member_bank_account_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_update_member_bank_account_res.result {
          message
            .respond(
              bpr::c2se::member_bank_account::UpdateMemberBankAccountResponse {
                error: None,
                result: Some(
                  bpr::c2se::member_bank_account::update_member_bank_account_response::Result {
                    member_bank_account: r.member_bank_account,
                  },
                ),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::member_bank_account::UpdateMemberBankAccountResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn update_member_bank_account_for_exchange_password(
    &self,
  ) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::frontend::member_bank_account::SUBJECT_UPDATE_MEMBER_BANK_ACCOUNT_FOR_EXCHANGE_PASSWORD,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req = bpr::c2se::member_bank_account::UpdateMemberBankAccountForExchangePasswordRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let ss_update_member_bank_account_req = bpr::ss::member_bank_account::UpdateMemberBankAccountForExchangePasswordRequest {
          client: Some(client),
          request: Some(bpr::ss::member_bank_account::update_member_bank_account_for_exchange_password_request::Request{
            id: req.id,
            exchange_password: req.exchange_password,

          })
        };

        let ss_update_member_bank_account_res_msg = self
          .connection_broker
          .request(
            bpr::ss::member_bank_account::SUBJECT_UPDATE_MEMBER_BANK_ACCOUNT_FOR_EXCHANGE_PASSWORD,
            ss_update_member_bank_account_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_update_member_bank_account_res = bpr::ss::member_bank_account::UpdateMemberBankAccountForExchangePasswordResponse::decode(
          ss_update_member_bank_account_res_msg.data.as_slice(),
        )
        .map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        if let Some(e) = ss_update_member_bank_account_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_update_member_bank_account_res.result {
          message
            .respond(
              bpr::c2se::member_bank_account::UpdateMemberBankAccountForExchangePasswordResponse {
                error: None,
                result: Some(bpr::c2se::member_bank_account::update_member_bank_account_for_exchange_password_response::Result {
                }),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::member_bank_account::UpdateMemberBankAccountForExchangePasswordResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }
}
