use beteran_common_rust as bcr;
use beteran_protobuf_rust as bpr;
use prost::Message;

///
pub struct Service {
  connection_broker: nats::asynk::Connection,
  queue_broker: String,
}

impl std::fmt::Debug for Service {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("beteran-frontend-server-edge::member_referrer::service::Service")
      .finish()
  }
}

impl Service {
  ///
  pub fn new(connection_broker: nats::asynk::Connection, queue_broker: String) -> Service {
    Service {
      connection_broker,
      queue_broker,
    }
  }

  pub async fn subscribe(&self) -> std::result::Result<(), std::boxed::Box<dyn std::error::Error>> {
    futures::try_join!(self.get_member_referrer_by_code(),).map(|_| ())
  }

  fn get_client_in_header(
    &self,
    message: &nats::asynk::Message,
  ) -> Result<bpr::models::core::network::Client, bcr::error::rpc::Error> {
    match &message.headers {
      Some(headers) => {
        let client = match headers.get(bpr::c2se::core::network::HEADER_CLIENT) {
          Some(c) => {
            let msg = base64::decode(c).map_err(|e| {
              bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
                message: format!("invalid header: {}", e),
              })
            })?;
            bpr::models::core::network::Client::decode(msg.as_slice()).map_err(|e| {
              bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
                message: format!("invalid header: {}", e),
              })
            })?
          }
          None => {
            return Err(bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
              message: "invalid client information".to_string(),
            }));
          }
        };

        Ok(client)
      }
      None => Err(bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
        message: "invalid header".to_string(),
      })),
    }
  }

  async fn get_member_session_with_renewal(
    &self,
    client: &bpr::models::core::network::Client,
  ) -> Result<Option<bpr::models::member_session::MemberSession>, bcr::error::rpc::Error> {
    let ss_get_member_session_req = bpr::ss::member_session::GetMemberSessionWithRenewalRequest {
      client: Some(client.clone()),
      request: Some(bpr::ss::member_session::get_member_session_with_renewal_request::Request {}),
    };

    let ss_get_member_session_res_msg = self
      .connection_broker
      .request(
        bpr::ss::member_session::SUBJECT_GET_MEMBER_SESSION_WITH_RENEWAL,
        ss_get_member_session_req.encode_to_vec(),
      )
      .await
      .map_err(|e| {
        bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
          code: bpr::protobuf::rpc::Error::SERVER_00,
          message: format!("server {}", e),
          data: None,
        })
      })?;

    let ss_get_member_session_res =
      bpr::ss::member_session::GetMemberSessionWithRenewalResponse::decode(
        ss_get_member_session_res_msg.data.as_slice(),
      )
      .map_err(|e| {
        bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
          code: bpr::protobuf::rpc::Error::SERVER_00,
          message: format!("server {}", e),
          data: None,
        })
      })?;

    if let Some(e) = ss_get_member_session_res.error {
      return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
        code: bpr::protobuf::rpc::Error::SERVER_00,
        message: format!("server {}", e),
        data: None,
      }));
    }

    if let Some(r) = ss_get_member_session_res.result {
      return Ok(r.member_session);
    }

    Ok(None)
  }

  async fn get_member_referrer_by_code(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::frontend::member_referrer::SUBJECT_GET_MEMBER_REFERRER_BY_CODE,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req = bpr::c2se::member_referrer::GetMemberReferrerByCodeRequest::decode(
          message.data.as_slice(),
        )
        .map_err(|e| {
          bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
            message: format!("invalid request: {}", e),
          })
        })?;

        let ss_get_member_referrer_by_code_req =
          bpr::ss::member_referrer::GetMemberReferrerByCodeRequest {
            client: Some(client),
            request: Some(
              bpr::ss::member_referrer::get_member_referrer_by_code_request::Request {
                code: req.code,
              },
            ),
          };

        let ss_get_member_referrer_by_code_res_msg = self
          .connection_broker
          .request(
            bpr::ss::member_referrer::SUBJECT_GET_MEMBER_REFERRER_BY_CODE,
            ss_get_member_referrer_by_code_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_get_member_referrer_by_code_res =
          bpr::ss::member_referrer::GetMemberReferrerByCodeResponse::decode(
            ss_get_member_referrer_by_code_res_msg.data.as_slice(),
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        if let Some(e) = ss_get_member_referrer_by_code_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_get_member_referrer_by_code_res.result {
          message
            .respond(
              bpr::c2se::member_referrer::GetMemberReferrerByCodeResponse {
                error: None,
                result: Some(
                  bpr::c2se::member_referrer::get_member_referrer_by_code_response::Result {
                    member_referrer: r.member_referrer,
                  },
                ),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::member_referrer::GetMemberReferrerByCodeResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }
}
